package edu.ncsu.Chat;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.UnknownHostException;

import android.content.Context;
import android.net.DhcpInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Handler;
import android.util.Log;


public class ServerThread extends Thread {

	final static private String TAG = "Chat";
	final static boolean D = true;
	final static int PORT = 6666;

	Handler 		mHandler;      // Handler for messages in the main thread 
	Context 		mContext;	   // Context to the application (for getting ip Addresses)
	DatagramSocket 	serverSocket;  // Socket used both for sending and receiving 
	boolean 		socketOK=true; // True as long as we don't get socket errors
	
	InetAddress 	myBcastIPAddress; 		// my broadcast IP addresses
	InetAddress 	myIPAddress; 			// my IP addresses

	// Constructor - make a new datagram socket
	public ServerThread(Context currentContext,	Handler handler){

		mContext = currentContext;
		mHandler = handler;
		
		try{ // Let's open an UDP socket - we'll receive the messages on this socket
			serverSocket = new DatagramSocket(PORT);
			serverSocket.setBroadcast(true);
		} catch(Exception e){
			Log.e(TAG,"Cannot open socket"+e.getMessage());
			socketOK = false;
			return;
		}
		
		try{
			getMyWiFiBcastAndIPAddress();
			Log.i(TAG,"My IP address:"+myIPAddress);
			Log.i(TAG,"My Broadcast IP address:"+myBcastIPAddress);
		}catch(Exception e){
			Log.e(TAG,"Cannot get my own Broadcast IP address");
		}
	}
	
	
	// Method for closing the socket before exiting application
	public void closeSocket(){
		serverSocket.close();
	}
	
	
	// If the socket is OK, then it's running
	boolean socketIsOK(){
	  return socketOK;
	}
	
	// What the thread does when started
	@Override
	public void run(){

		byte[] receiveData = new byte[1024]; 

		while(socketOK) { 
			
			DatagramPacket receivePacket = 
			    new DatagramPacket(receiveData, receiveData.length); 

			try{
				serverSocket.receive(receivePacket); 
		        Log.i(TAG,"Received a packet");
		
		        InetAddress sourceIPAddress = receivePacket.getAddress();
		        Log.d(TAG,"Source IP Address: "+sourceIPAddress);
		        if(!sourceIPAddress.equals(myIPAddress)){// it's from somebody else
		        	String sentence = new String(receivePacket.getData(),0,receivePacket.getLength()); 
		        	mHandler.obtainMessage(Chat.PACKET_CAME,sentence).sendToTarget();    
		        	Log.i(TAG,"Received sentence: "+sentence);
		        }// else do nothing
			} catch (Exception e){
				Log.e(TAG,"Problems receiving packet: "+e.getMessage());
				socketOK = false;
			} //end catch
			
		}// while(socketOK) 
		
	}// end run()


	// Sends an UDP packet at the broadcast address
	public void sendMessage(String msg) throws IOException {
		
		byte[] sendData  = new byte[1024]; 

		sendData = msg.getBytes(); 

		DatagramPacket sendPacket = 
			new DatagramPacket(sendData, sendData.length, myBcastIPAddress, PORT); 

		serverSocket.send(sendPacket);
        Log.i(TAG,"Sent packet: "+msg);

	}
	
	
    private void getMyWiFiBcastAndIPAddress() throws UnknownHostException{

        WifiManager mWifi = (WifiManager) (mContext.getSystemService(Context.WIFI_SERVICE));
        WifiInfo info = mWifi.getConnectionInfo();
        if(info==null){
            if(D) Log.e(TAG,"Cannot Get WiFi Info");
            return;
        }
        else{
        	if(D) Log.d(TAG,"\n\nWiFi Status: " + info.toString());
        }
		  
  	  // DhcpInfo  is a simple object for retrieving the results of a DHCP request
        DhcpInfo dhcp = mWifi.getDhcpInfo(); 
        if (dhcp == null) { 
          Log.d(TAG, "Could not get dhcp info"); 
          return; 
        } 


        int myIntegerIPAddress = dhcp.ipAddress;

        byte[] quads = new byte[4]; 
        for (int k = 0; k < 4; k++) 
           quads[k] = (byte) ((myIntegerIPAddress>> k * 8) & 0xFF);

        myIPAddress = InetAddress.getByAddress(quads);

        
        int myIntBroadcast = (dhcp.ipAddress & dhcp.netmask) | ~dhcp.netmask; 
        for (int k = 0; k < 4; k++) 
          quads[k] = (byte) ((myIntBroadcast >> k * 8) & 0xFF);
        
        // Returns the InetAddress corresponding to the array of bytes. 
        myBcastIPAddress=InetAddress.getByAddress(quads); 
    	
    }
    	
	
}// end class
